#include "Stage.h"
#include "Stages.h"
#include "Main.h"
#include "Vram.h"
#include "VVram.h"
#include "Sprite.h"
#include "Chars.h"
#include "Status.h"
#include "CopyMemory.h"
#include "Man.h"
#include "Monster.h"
#include "Destruction.h"
#include "Background.h"

extern void _deb();

ptr<Stage> pStage;
ptr<byte> pStageBytes;
byte[ColumnCount / ColumnsPerByte * FloorCount] CellMap;

static const byte[] UpperBackgrounds = {
    BG_Space, BG_Space, BG_Space,
    BG_Space, BG_Space, BG_Space,
    BG_Ladder, BG_Ladder, BG_Ladder,
    BG_Space, BG_ItemUpper, BG_ItemLower,
};
static const byte[] LowerBackgrounds = {
    BG_Space, BG_Space,
    BG_Space, BG_CrackedFloor,
    BG_Ladder, BG_Ladder,
    BG_Space, BG_HardFloor,
};

void InitStage() 
{
    ptr<Stage> p;
    byte i, j;
    p = Stages;
    i = 0;
    j = 0;
    while (i < CurrentStage) {
        ++p;
        ++i;
        ++j;
        if (j >= StageCount) {
            p = Stages;
            j = 0;
        }
    }
    pStage = p;
    pStageBytes = pStage->bytes;
}


static void DrawCell(byte x, byte y, byte b)
{
    {
        ptr<byte> p;
        byte offset;
        offset = (b & 0xc) >> 1;
        offset = (offset >> 1) + offset;
        p = UpperBackgrounds + offset;
        repeat (3) {
            if (y < VVramHeight) {
                SetBackground(x, y, *p);
                ++p;
            }
            ++y;
        }
    }
    --y;
    {
        ptr<byte> p;
        byte offset;
        offset = b & 0x3;
        offset <<= 1; 
        p = LowerBackgrounds + offset;
        {
            if (y < VVramHeight) {
                byte c;
                c = *p; ++p;
                if (c != BG_Space) {
                    SetBackground(x, y, c);
                }
            }
            ++y;
        }
        {
            if (y < VVramHeight) {
                SetBackground(x, y, *p);
                ++p;
            }
            ++y;
        }
    }
}

void InitTrying()
{
    {
        byte i;
        StageTime = 25;
        i = pStage->itemCount;
        do {
            StageTime += 5;
            --i;
        } while (i != 0);
    }
    ItemCount = pStage->itemCount;
    HideAllSprites();
    ClearScreen();
    PrintStatus();

    CopyMemory(CellMap, pStageBytes, ColumnCount / ColumnsPerByte * FloorCount);
    {
        ptr<byte> pByte;
        byte y;
        pByte = pStageBytes;
        y = 0;
        repeat (FloorCount) {
            byte x;
            x = 0;
            repeat (ColumnCount / ColumnsPerByte) {
                byte b;
                b = *pByte; ++pByte;
                repeat (ColumnsPerByte) {
                    DrawCell(x, y, b);
                    b >>= (8 / ColumnsPerByte);
                    ++x;
                }
            }
            y += FloorHeight;
        }
    }
    InitMan();
    InitMonsters();
    InitDestructions();
    // UpdateSprites();
    VVramChanged = true;
    DrawAll();
}


byte GetCell(byte column, byte floor)
{
    byte b;
    b = *CellMapPtr(column, floor);
    if ((column & 1) == 0) {
        return b & 0x0f;
    }
    else {
        return b >> 4;
    }
}


void SetCell(byte column, byte floor, byte cell)
{
    ptr<byte> pCell;
    byte mask;
    pCell = CellMapPtr(column, floor);
    if ((column & 1) == 0) {
        mask = 0xf0;
        cell &= 0x0f;
    }
    else {
        mask = 0x0f;
        cell <<= 4;
    }
    *pCell = (*pCell & mask) | cell;
}