#include "Status.h"
#include "Vram.h"
#include "Stage.h"
#include "Main.h"
#include "Sprite.h"
#include "Print.h"
#include "Chars.h"
#include "CopyMemory.h"

constexpr byte Pattern_Remain = Pattern_Man;

extern byte[] Chars;

static void PrintS(ptr<byte> pVram, ptr<byte> p)
{
    byte c;
    Locate(pVram);
    while ((c = *p) != 0) {
        PrintC(c);
        ++p;
    }
    EndPrint();
}

static void PrintRemain(ptr<byte> pVram, byte c)
{
    ptr<byte> pPattern;
    pPattern = Chars + Pattern_Remain * PatternSize;
    repeat (2) {
        VLocate(pVram, 0, c);
        repeat (2) {
            VPut(pPattern);
            pPattern += PatternSize;
        }
        pVram += VramRowSize;
        c += 2;
    }
}

static void EraseRemain(ptr<byte> pVram)
{
    repeat (2) {
        repeat (2) {
            *pVram = Char_Space;
            ++pVram;
        }
        pVram += VramRowSize - 2;
    }
}


void PrintStatus() 
{
    CurrentChar(Char_Score);

    PrintS(Vram + 2, "SCORE");
    PrintS(Vram + 6, "HI-SCORE");
    PrintS(Vram + 11, "STAGE");
    PrintByteNumber2(Vram + VramRowSize + 12, CurrentStage + 1);
    EndPrint();
    PrintS(Vram + 15, "TIME");

    {
        ptr<byte> pVram;
        byte c;
        pVram = Vram + 19;
        c = Char_Remain;
        if (RemainCount > 1) {
            byte i;
            i = RemainCount - 1;
            if (i > 2) {
                PrintRemain(pVram, c);
                pVram += 2;
                EraseRemain(pVram);
                CurrentChar(Char_RemainValue);
                Locate(Vram + VramRowSize + 21);
                PrintC(i + 0x30);
                EndPrint();
            }
            else {
                while (i > 0) {
                    PrintRemain(pVram, c); 
                    c += 4;
                    pVram += 2;
                    --i;
                }
                EraseRemain(pVram);
            }
        }
    }
    // FillMemory(ColorMap + VramRowSize * 0 + 19, 5, 3);
    // FillMemory(ColorMap + VramRowSize * 1 + 19, 5, 3);
 
    PrintScore();
    PrintTime();
}

void PrintScore()
{
    CurrentChar(Char_ScoreValue);

    PrintNumber5(Vram + VramRowSize + 1, Score);
    PrintC('0');
    EndPrint();

    PrintNumber5(Vram + VramRowSize + 7, HiScore);
    PrintC('0');
    EndPrint();
}

void PrintTime()
{
    CurrentChar(Char_TimeValue);
    PrintByteNumber3(Vram + VramRowSize + 15, StageTime);
    EndPrint();
}

void PrintGameOver()
{
    CurrentChar(Char_Message);
    PrintS(Vram + VramRowSize * 12 + 10, "GAME OVER");
}

void PrintTimeUp() 
{
    CurrentChar(Char_Message);
    PrintS(Vram + VramRowSize * 12 + 10, " TIME UP");
}

void Title()
{
    constexpr byte LogoLength = 6;
    static const byte[] TitleBytes = {
        //	C
        0x00, 0x0e, 0x05, 0x0b, 
        0x0c, 0x03, 0x00, 0x00, 
        0x04, 0x0b, 0x00, 0x0a, 
        0x00, 0x04, 0x05, 0x01, 
        //	R
        0x0c, 0x07, 0x05, 0x0b, 
        0x0c, 0x03, 0x08, 0x0f, 
        0x0c, 0x07, 0x0f, 0x02, 
        0x04, 0x01, 0x04, 0x05, 
        //	A
        0x00, 0x0e, 0x0d, 0x02, 
        0x0c, 0x03, 0x00, 0x0f, 
        0x0c, 0x07, 0x05, 0x0f, 
        0x04, 0x01, 0x00, 0x05, 
        //	C
        0x00, 0x0e, 0x05, 0x0b, 
        0x0c, 0x03, 0x00, 0x00, 
        0x04, 0x0b, 0x00, 0x0a, 
        0x00, 0x04, 0x05, 0x01, 
        //	K
        0x0c, 0x03, 0x08, 0x07, 
        0x0c, 0x0b, 0x07, 0x00, 
        0x0c, 0x03, 0x0d, 0x02, 
        0x04, 0x01, 0x00, 0x05, 
        //	Y
        0x0c, 0x03, 0x0c, 0x03, 
        0x04, 0x0b, 0x0e, 0x01, 
        0x00, 0x0c, 0x03, 0x00, 
        0x00, 0x04, 0x01, 0x00, 
    };

    ClearScreen(); 
    HideAllSprites();
    PrintStatus();
    {
        constexpr byte left = (32 - 4 * LogoLength) / 2;
        ptr<byte> p;
        ptr<byte> pVram;
        byte c;

        pVram = Vram + VramRowSize * 7 + left * 3 / 4;
        c = Char_Logo;
        p = TitleBytes;
        repeat (LogoLength) {
            repeat (4) {
                VLocate(pVram, 0, c);
                repeat (4) {
                    ptr<byte> pPattern;
                    pPattern = Chars + (((word)*p) << 3);
                    VPut(pPattern);
                    ++p;
                }
                pVram += VramRowSize;
                c += 3;
            }
            pVram += 3 - VramRowSize * 4;
        }
        FillMemory(ColorMap + VramRowSize * 7, 4 * VramRowSize, 3);
    }
    CurrentChar(Char_Message2);
    PrintS(Vram + VramRowSize * 19 + 9, "PUSH SPACE KEY");
    PrintS(Vram + VramRowSize * 20 + 9, "OR 'Z' KEY");
    PrintS(Vram + VramRowSize * 23 + 18, "INUFUTO 2023");

    // {
    //     ptr<byte> pVram;
    //     byte c;
    //     pVram = Vram + 24*12;
    //     c = Char_Cell;
    //     repeat(24) {
    //         *pVram = c;
    //         ++c;
    //         ++pVram;
    //     }
    //     repeat(24) {
    //         *pVram = 1;
    //         ++pVram;
    //     }
    //     repeat(24) {
    //         *pVram = c;
    //         ++c;
    //         ++pVram;
    //     }
    //     repeat(24) {
    //         *pVram = 1;
    //         ++pVram;
    //     }
    // }
}